@@ -39,7 +39,7 @@ struct _TaskTitlePrivate {
     GtkWidget *button_image;
     GdkPixbuf *quit_icon;
     gboolean mouse_in_close_button;
-    WindowPickerApplet *windowPickerApplet;
+    WpApplet *windowPickerApplet;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (TaskTitle, task_title, GTK_TYPE_EVENT_BOX);
@@ -185,6 +185,36 @@ static void hide_title(TaskTitle *title) {
     gtk_widget_set_tooltip_text (GTK_WIDGET (title), NULL);
     gtk_widget_hide (title->priv->grid);
 }
+
+static gboolean
+is_desktop_visible (void)
+{
+    WnckScreen *screen;
+    GList *windows;
+    GList *w;
+
+    screen = wnck_screen_get_default ();
+    windows = wnck_screen_get_windows (screen);
+
+    for (w = windows; w; w = w->next)
+    {
+        WnckWindow *window;
+
+        window = WNCK_WINDOW (w->data);
+
+        if (WNCK_IS_WINDOW (window) == FALSE)
+            continue;
+
+        if (wnck_window_get_window_type (window) == WNCK_WINDOW_DESKTOP)
+            continue;
+
+        if (wnck_window_is_minimized (window) == FALSE)
+            return FALSE;
+    }
+
+    return TRUE;
+}
+
 static void
 on_active_window_changed (WnckScreen *screen,
                           WnckWindow *old_window,
@@ -204,7 +234,7 @@ on_active_window_changed (WnckScreen *screen,
         if(type == WNCK_WINDOW_DESKTOP) {
             /* The current window is the desktop so we show the home title if
              *  the user has configured this, otherwise we hide the title */
-            if (window_picker_applet_get_show_home_title (priv->windowPickerApplet)) {
+            if (wp_applet_get_show_home_title (priv->windowPickerApplet)) {
                 show_home_title(title);
             } else {
                 hide_title (title);
@@ -221,7 +251,7 @@ on_active_window_changed (WnckScreen *screen,
         {
             return;
         } else { //for all other types
-            if(wnck_window_is_maximized (act_window) && window_picker_applet_get_show_application_title (priv->windowPickerApplet)) {
+            if(wnck_window_is_maximized (act_window) && wp_applet_get_show_application_title (priv->windowPickerApplet)) {
                 //show normal title of window
                 gtk_label_set_text (GTK_LABEL (priv->label),
                     wnck_window_get_name (act_window));
@@ -241,8 +271,7 @@ on_active_window_changed (WnckScreen *screen,
             }
         }
     } else { //its not a window
-        if (task_list_get_desktop_visible (TASK_LIST (window_picker_applet_get_tasks (priv->windowPickerApplet)))
-                && window_picker_applet_get_show_home_title (priv->windowPickerApplet))
+        if (is_desktop_visible () && wp_applet_get_show_home_title (priv->windowPickerApplet))
         {
             show_home_title(title);
         } else { //reset the task title and hide it
@@ -405,14 +434,15 @@ task_title_setup (TaskTitle *title)
 }
 
 static void
-task_title_finalize (GObject *object)
+task_title_dispose (GObject *object)
 {
     TaskTitle *title = TASK_TITLE (object);
 
+    g_signal_handlers_disconnect_by_func (title->priv->screen, on_active_window_changed, title);
     disconnect_window (title);
-    g_object_unref (title->priv->quit_icon);
 
-    G_OBJECT_CLASS (task_title_parent_class)->finalize (object);
+    g_clear_object (&title->priv->quit_icon);
+    G_OBJECT_CLASS (task_title_parent_class)->dispose (object);
 }
 
 static GObject *
@@ -443,11 +473,11 @@ task_title_class_init (TaskTitleClass *klass)
     GObjectClass *obj_class = G_OBJECT_CLASS (klass);
 
     obj_class->constructor = task_title_constructor;
-    obj_class->finalize = task_title_finalize;
+    obj_class->dispose = task_title_dispose;
 }
 
 GtkWidget *
-task_title_new (WindowPickerApplet *windowPickerApplet)
+task_title_new (WpApplet *windowPickerApplet)
 {
     TaskTitle *title = g_object_new (TASK_TYPE_TITLE,
                                    "border-width", 0,
@@ -457,7 +487,7 @@ task_title_new (WindowPickerApplet *windowPickerApplet)
 
     title->priv->windowPickerApplet = windowPickerApplet;
 
-    if (window_picker_applet_get_show_home_title (title->priv->windowPickerApplet)) {
+    if (wp_applet_get_show_home_title (title->priv->windowPickerApplet)) {
         gtk_widget_set_state_flags (GTK_WIDGET (title), GTK_STATE_FLAG_ACTIVE, TRUE);
     } else {
         gtk_widget_hide (title->priv->grid);
